@@ -23,7 +23,7 @@ BEGIN {
     use Exporter ();
     our ( $VERSION, @EXPORT_OK, %EXPORT_TAGS );
 
-    $VERSION = '2.8.1';
+    $VERSION = '2.9.0';
 
     use base qw(Exporter);
     %EXPORT_TAGS = ();    # eg: TAG => [ qw!name1 name2! ],
@@ -43,6 +43,20 @@ sub new {
     return bless $self, $class;
 }
 
+sub valid {
+    my ( $class, $json ) = @_;
+    my $self = $class->new($json);
+    my $iter = $self->get_bbox_iter();
+    while ( my $bbox = $iter->() ) {
+        my ( $x1, $y1, $x2, $y2 ) = @{ $bbox->{bbox} };
+        if ( $bbox->{type} eq 'page' ) {
+            if ( $x2 == 0 or $y2 == 0 ) { return FALSE }
+            return TRUE;
+        }
+    }
+    return FALSE;
+}
+
 sub json {
     my ($self) = @_;
     my @boxes;
@@ -55,7 +69,7 @@ sub json {
 
 sub from_hocr {
     my ( $self, $hocr ) = @_;
-    if ( $hocr !~ /<body>[\s\S]*<\/body>/xsm ) { return }
+    if ( not defined $hocr or $hocr !~ /<body>[\s\S]*<\/body>/xsm ) { return }
     my $box_tree = _hocr2boxes($hocr);
     _prune_empty_branches($box_tree);
     if ( $#{$box_tree} > $EMPTY_LIST ) {
@@ -523,6 +537,63 @@ EOS
     return $string;
 }
 
+sub crop {
+    my ( $self, $x, $y, $w, $h ) = @_;
+    my $i = 0;
+    while ( $i <= $#{$self} ) {
+        my $bbox = $self->[$i];
+        my ( $text_x1, $text_y1, $text_x2, $text_y2 ) = @{ $bbox->{bbox} };
+        ( $text_x1, $text_x2 ) = _crop_axis( $text_x1, $text_x2, $x, $x + $w );
+        ( $text_y1, $text_y2 ) = _crop_axis( $text_y1, $text_y2, $y, $y + $h );
+
+        # cropped outside box, so remove box
+        if ( not defined $text_x1 or not defined $text_y1 ) {
+            splice @{$self}, $i, 1;
+            next;
+        }
+
+        # update box
+        $bbox->{bbox} = [ $text_x1, $text_y1, $text_x2, $text_y2 ];
+        $i++;
+    }
+    return $self;
+}
+
+# More trouble that it's worth to refactor this as a dispatch table, as it would
+# require converting the inequalities into a string of binaries, which would be
+# less easy to understand. In this case the if-else cascade is better.
+
+sub _crop_axis {
+    my ( $text1, $text2, $crop1, $crop2 ) = @_;
+    if ( $text1 > $crop2 or $text2 < $crop1 ) { return }
+
+    # crop inside edges of box
+    if ( $text1 <= $crop1 and $text2 >= $crop2 )
+    {    ## no critic (ProhibitCascadingIfElse)
+        $text1 = 0;
+        $text2 = $crop2 - $crop1;
+    }
+
+    # crop outside edges of box
+    elsif ( $text1 >= $crop1 and $text2 <= $crop2 ) {
+        $text1 -= $crop1;
+        $text2 -= $crop1;
+    }
+
+    # crop over 2nd edge of box
+    elsif ( $text2 >= $crop1 and $text2 <= $crop2 ) {
+        $text1 = 0;
+        $text2 -= $crop1;
+    }
+
+    # crop over 1st edge of box
+    elsif ( $text1 >= $crop1 and $text1 <= $crop2 ) {
+        $text1 -= $crop1;
+        $text2 = $crop2 - $crop1;
+    }
+    return $text1, $text2;
+}
+
 1;
 
 __END__
