@@ -7,15 +7,45 @@ use Carp;
 use Encode;
 use File::Temp;    # To create temporary files
 use File::Basename;
-use Gscan2pdf::Document;             # for slurp
+use Gscan2pdf::Document;    # for slurp
 use version;
 use English qw( -no_match_vars );    # for $PROCESS_ID
+use Gscan2pdf::Translation '__';     # easier to extract strings with xgettext
+use Locale::Language;
 
-our $VERSION = '2.8.1';
+our $VERSION = '2.9.0';
 my $EMPTY = q{};
 my $COMMA = q{,};
 
-my ( %languages, $installed, $setup, $version, $logger );
+my ( %languages, %installable_languages, $installed, $setup, $version,
+    $logger );
+
+# Taken from
+# https://github.com/tesseract-ocr/tesseract/blob/master/doc/tesseract.1.asc#languages
+my @installable_languages =
+  qw(afr amh ara asm aze aze-cyrl bel ben bod bos bre bul cat ceb ces chi-sim chi-sim-vert chi-tra chi-tra-vert chr cos cym dan dan-frak deu deu-frak div dzo ell eng enm epo equ est eus fao fas fil fin fra frk frm fry gla gle gle-uncial glg grc guj hat heb hin hrv hun hye iku ind isl ita ita-old jav jpn jpn-vert kan kat kat-old kaz khm kir kmr kor kor-vert lao lat lav lit ltz mal mar mkd mlt mon mri msa mya nep nld nor oci ori pan pol por pus que ron rus san sin slk slk-frak slv snd spa spa_old sqi srp srp_latn sun swa swe swe-frak syr tam tat tel tgk tgl tha tir ton tur uig ukr urd uzb uzb_cyrl vie yid yor);
+my %non_iso639_3 = (
+    'aze-cyrl'     => 'Azerbaijani (Cyrillic)',
+    'chi-sim'      => 'Simplified Chinese',
+    'chi-sim-vert' => 'Chinese - Simplified (vertical)',
+    'chi-tra'      => 'Traditional Chinese',
+    'chi-tra-vert' => 'Traditional Chinese (vertical)',
+    'dan-frak'     => 'Danish (Fraktur)',
+    'deu-frak'     => 'German (Fraktur)',
+    equ            => 'equations',
+    'gle-uncial'   => 'Irish (Uncial)',
+    'ita-old'      => 'Italian - Old',
+    'jpn-vert'     => 'Japanese (vertical)',
+    'kat-old'      => 'Old Georgian',
+    'kor-vert'     => 'Korean (vertical)',
+    osd            => 'Orientation, script, direction',
+    'slk-frak'     => 'Slovak (Fraktur)',
+    spa_old        => 'Spanish (Castilian - Old)',
+    srp_latn       => 'Serbian - Latin',
+    'swe-frak'     => 'Swedish (Fraktur)',
+    uzb_cyrl       => 'Uzbek - Cyrilic',
+);
+my %non_iso639_1 = ( zh => 'chi-sim', );
 
 sub setup {
     ( my $class, $logger ) = @_;
@@ -36,7 +66,7 @@ sub setup {
     elsif ( $out =~ /^tesseract[ ]([\d.]+)/xsm ) {
         $version = $1;
     }
-    if ( not $version ) { return }
+    if ( not $version )                 { return }
     if ( $version !~ /^\d+[.]\d+$/xsm ) { $version = 'v' . $version }
     $version = version->parse($version);
     if ( $version > version->parse('v3.02.00') ) {
@@ -52,53 +82,6 @@ sub setup {
 
 sub languages {
     if ( not %languages ) {
-        my %iso639 = (
-            ara        => 'Arabic',
-            bul        => 'Bulgarian',
-            cat        => 'Catalan',
-            ces        => 'Czech',
-            chr        => 'Cherokee',
-            chi_tra    => 'Chinese (Traditional)',
-            chi_sim    => 'Chinese (Simplified)',
-            dan        => 'Danish',
-            'dan-frak' => 'Danish (Fraktur)',
-            deu        => 'German',
-            'deu-f'    => 'German (Fraktur)',
-            'deu-frak' => 'German (Fraktur)',
-            ell        => 'Greek',
-            eng        => 'English',
-            fin        => 'Finish',
-            fra        => 'French',
-            heb        => 'Hebrew',
-            hin        => 'Hindi',
-            hun        => 'Hungarian',
-            ind        => 'Indonesian',
-            ita        => 'Italian',
-            jpn        => 'Japanese',
-            kor        => 'Korean',
-            lav        => 'Latvian',
-            lit        => 'Lituanian',
-            nld        => 'Dutch',
-            nor        => 'Norwegian',
-            pol        => 'Polish',
-            por        => 'Portuguese',
-            que        => 'Quechua',
-            ron        => 'Romanian',
-            rus        => 'Russian',
-            slk        => 'Slovak',
-            'slk-frak' => 'Slovak (Fraktur)',
-            slv        => 'Slovenian',
-            spa        => 'Spanish',
-            srp        => 'Serbian (Latin)',
-            swe        => 'Swedish',
-            'swe-frak' => 'Swedish (Fraktur)',
-            tha        => 'Thai',
-            tlg        => 'Tagalog',
-            tur        => 'Turkish',
-            ukr        => 'Ukranian',
-            vie        => 'Vietnamese',
-        );
-
         my @codes;
         my ( undef, $out, $err ) =
           Gscan2pdf::Document::exec_command( [ 'tesseract', '--list-langs' ] );
@@ -106,20 +89,74 @@ sub languages {
         if ( $codes[0] =~ /^List[ ]of[ ]available[ ]languages/xsm ) {
             shift @codes;
         }
-
-        for (@codes) {
-            $logger->info("Found tesseract language $_");
-            if ( defined $iso639{$_} ) {
-                $languages{$_} = $iso639{$_};
+        for my $code (@codes) {
+            my $name = code2language( $code, 'term' );
+            if ( not defined $name ) {
+                $name = $non_iso639_3{$code};
             }
-            else {
-                $languages{$_} = $_;
+            if ( not defined $name ) {
+                $name = $code;
             }
+            $logger->info("Found tesseract language $code ($name)");
+            $languages{$code} = $name;
         }
     }
     return \%languages;
 }
 
+sub installable_languages {
+    if ( not %installable_languages ) {
+        %installable_languages = %non_iso639_3;
+        for my $code (@installable_languages) {
+            my $language = code2language( $code, 'term' );
+            if ( not defined $language ) {
+                $language = $non_iso639_3{$code};
+            }
+            $installable_languages{$code} = $language;
+        }
+    }
+    return \%installable_languages;
+}
+
+sub _iso639_1to3 {
+    my ($code1) = @_;
+    my $code3 = $non_iso639_1{$code1};
+    if ($code3) { return $code3 }
+    return language_code2code( $code1, 'alpha-2', 'term' );
+}
+
+sub locale_installed {
+    my ( $class, $locale ) = @_;
+    my $code1     = lc substr $locale, 0, 2;
+    my $code3     = _iso639_1to3($code1);
+    my $languages = languages();
+    if ( not defined $code3 ) {
+        return
+          sprintf( __("You are using locale '%s'."), $locale ) . q{ }
+          . __(
+'gscan2pdf does not currently know which tesseract language package would be necessary for that locale.'
+          )
+          . q{ }
+          . __('Please contact the developers to add support for that locale.');
+    }
+    if ( defined $languages->{$code3} ) {
+        return 1;
+    }
+    $languages = installable_languages();
+    if ( defined $languages->{$code3} ) {
+        return
+          sprintf( __("You are using locale '%s'."), $locale ) . q{ }
+          . sprintf __(
+"Please install tesseract package 'tesseract-ocr-%s' and restart gscan2pdf for OCR for %s with tesseract."
+          ), $code3, $languages->{$code3};
+    }
+    return
+        sprintf( __("You are using locale '%s'."), $locale ) . q{ }
+      . sprintf __('There is no tesseract package for %s'),
+      code2language( $code3, 'term' ) . '. '
+      . 'If this is in error, please contact the gscan2pdf developers.';
+}
+
 sub hocr {
     my ( $class, %options ) = @_;
     my ( $tif, $cmd, $name, $path, $txt );
